//AsterPong - An Astroids/Hockey/Pong type game Version 0.04
//(Volume I of Ciphersoft Sports)
//Copyright 2000 Justin Todd

#define	FRICTION	0.99	//Universal friction
#define NET_SIZE	45		//Net size in pixels
#define PADDLE_SIZE	20		//Length of the paddles
#define B_COLOR		1		//Color of all the boundry lines etc...
#define NET_COLOR	11		//Color of the net
#define ROTATE_RATE	5		//Rotation speed of the object
#define SPEED		0.2		//Speed of the paddle ships

#include"allegro.h"
#include<stdio.h>			//sprintf()

BITMAP *virtScreen;			//virtual screen for smooth animation

typedef struct{				//Structure containing general data for an object
	float x,y;				//Actual x and y location
	float xvel,yvel;		//x and y velocity of the object
	int sx,sy;				//x and y location on the screen
	float rxvel,ryvel;		//Rotation velocity
	fixed angle;			//View angle of the object
}OBJECT;

double radians;

OBJECT ship;
OBJECT puck;

void Init(){				//Initialize Allegro
	allegro_init();
	install_keyboard();
	set_gfx_mode(GFX_AUTODETECT,640,480,0,0);					//Set graphics mode to 640x480
	(BITMAP *)virtScreen=create_bitmap(SCREEN_W,SCREEN_H);		//Create (BITMAP *)virtScreen screen double buffer animation
}

void Game_Init(){												//Set the ojects on the screen
	ship.x=SCREEN_W/2-5;
	ship.y=SCREEN_H/2;
	puck.x=SCREEN_W/2;
	puck.y=SCREEN_H/2;
}

void Input(){													//Process inputs from the keyboard
	if(key[KEY_LEFT])  ship.angle-=itofix(ROTATE_RATE);
	if(key[KEY_RIGHT]) ship.angle+=itofix(ROTATE_RATE);
	if(key[KEY_UP]){
		ship.xvel+=fixtof(fsin(ship.angle))*SPEED;
		ship.yvel-=fixtof(fcos(ship.angle))*SPEED;
	}
}

void Update_Object(OBJECT *object){
	//Apply friction to objects
	object->xvel*=FRICTION;
	object->yvel*=FRICTION;	

	//Move the objects according to their x-y velocities
	object->x+=object->xvel;
	object->y+=object->yvel;

	//Get the screen coordinate (int) from the actual coordinate (float)
	object->sx=object->x;
	object->sy=object->y;

	//Keep objects within the boundries
	if(object->x<0) { object->x=0; object->xvel*=-1; }
	if(object->y<0) { object->y=0; object->yvel*=-1; }
	if(object->x>SCREEN_W) { object->x=SCREEN_W; object->xvel*=-1; }
	if(object->y>SCREEN_H) { object->y=SCREEN_H; object->yvel*=-1; }
}

void Draw_Objects(){
	int i;
	for(i=0;i<PADDLE_SIZE;i++)									//Draws the paddle (line) using sine and cosine of the view angle
		putpixel((BITMAP *)virtScreen,ship.x+i*fixtof(fsin(ship.angle)),ship.y-i*fixtof(fcos(ship.angle)),14);
	circlefill((BITMAP *)virtScreen,puck.x,puck.y,3,15);		//Draw the puck
}

void Draw_Playing_Field(){
	char string[80];

	vline((BITMAP *)virtScreen,SCREEN_W/2,0,SCREEN_H,B_COLOR);							//Line dividing the 2 sides
	vline((BITMAP *)virtScreen,30,0,SCREEN_H,B_COLOR);									//Left goal line
	vline((BITMAP *)virtScreen,SCREEN_W-30,0,SCREEN_H,B_COLOR);							//Right goal line	
	circle((BITMAP *)virtScreen,SCREEN_W/2,SCREEN_H/2,50,B_COLOR);						//Large center circle
	circlefill((BITMAP *)virtScreen,SCREEN_W/2,SCREEN_H/2,2,B_COLOR);					//Small center circle
	circlefill((BITMAP *)virtScreen,30,(SCREEN_H+NET_SIZE)/2,3,NET_COLOR);				//Left-top post
	circlefill((BITMAP *)virtScreen,30,(SCREEN_H-NET_SIZE)/2,3,NET_COLOR);				//Left-bottom post
	circlefill((BITMAP *)virtScreen,SCREEN_W-30,(SCREEN_H+NET_SIZE)/2,3,NET_COLOR);		//Right-top post
	circlefill((BITMAP *)virtScreen,SCREEN_W-30,(SCREEN_H-NET_SIZE)/2,3,NET_COLOR);		//Right-bottom post
	rect((BITMAP *)virtScreen,0,0,SCREEN_W-1,SCREEN_H-1,B_COLOR);						//Outer boundry
	sprintf(string,"%f", fixtof(ship.angle));
	textout((BITMAP *)virtScreen,font,string,1,1,15);

}

void Update_Graphics(){
	vsync();
	if(key[KEY_S]) save_bitmap("scrsht.bmp",(BITMAP *)virtScreen,NULL);
	blit((BITMAP *)virtScreen,screen,0,0,0,0,SCREEN_W,SCREEN_H);
	clear((BITMAP *)virtScreen);
}

void Check_Collision(){
	int i;
	radians=ship.angle*0.01745;
	for(i=0;i<PADDLE_SIZE;i++){
		if(getpixel(screen,ship.x+i*fixtof(fsin(ship.angle)),ship.y-i*fixtof(fcos(ship.angle)))==15){
			puck.xvel=ship.xvel*1.5;
			puck.yvel=ship.yvel*1.5;
		}
	}
}

int main(){
	Init();
	Game_Init();
	while(!key[KEY_ESC]){																//Do the following until the escape key is hit
		Input();
		Check_Collision();
		Update_Object(&ship);
		Update_Object(&puck);
		Draw_Playing_Field();
		Draw_Objects();	
		Update_Graphics();
	}
	destroy_bitmap((BITMAP *)virtScreen);
	allegro_exit();
	return 1;
} END_OF_MAIN()